"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessagePickupModule = void 0;
const core_1 = require("@credo-ts/core");
const FeatureRegistry_1 = require("../../FeatureRegistry");
const MessageHandlerRegistry_1 = require("../../MessageHandlerRegistry");
const MessagePickupApi_1 = require("./MessagePickupApi");
const MessagePickupModuleConfig_1 = require("./MessagePickupModuleConfig");
const protocol_1 = require("./protocol");
const services_1 = require("./services");
const storage_1 = require("./storage");
class MessagePickupModule {
    constructor(config) {
        // Infer Api type from the config
        this.api = MessagePickupApi_1.MessagePickupApi;
        this.config = new MessagePickupModuleConfig_1.MessagePickupModuleConfig({
            ...config,
            protocols: config?.protocols ?? [new protocol_1.V1MessagePickupProtocol(), new protocol_1.V2MessagePickupProtocol()],
        });
    }
    /**
     * Registers the dependencies of the message pickup answer module on the dependency manager.
     */
    register(dependencyManager) {
        // Config
        dependencyManager.registerInstance(MessagePickupModuleConfig_1.MessagePickupModuleConfig, this.config);
        // Services
        dependencyManager.registerSingleton(services_1.MessagePickupSessionService);
        // Message Pickup queue: use provided one or in-memory one if no injection symbol is yet defined
        if (this.config.messagePickupRepository) {
            dependencyManager.registerInstance(core_1.InjectionSymbols.MessagePickupRepository, this.config.messagePickupRepository);
        }
        else {
            if (!dependencyManager.isRegistered(core_1.InjectionSymbols.MessagePickupRepository)) {
                dependencyManager.registerSingleton(core_1.InjectionSymbols.MessagePickupRepository, storage_1.InMemoryMessagePickupRepository);
            }
        }
    }
    async initialize(agentContext) {
        // Protocol needs to register feature registry items and handlers
        const messageHandlerRegistry = agentContext.dependencyManager.resolve(MessageHandlerRegistry_1.MessageHandlerRegistry);
        const featureRegistry = agentContext.dependencyManager.resolve(FeatureRegistry_1.FeatureRegistry);
        for (const protocol of this.config.protocols) {
            protocol.register(messageHandlerRegistry, featureRegistry);
        }
        const messagePickupSessionService = agentContext.dependencyManager.resolve(services_1.MessagePickupSessionService);
        messagePickupSessionService.start(agentContext);
    }
}
exports.MessagePickupModule = MessagePickupModule;
//# sourceMappingURL=MessagePickupModule.js.map